#!/usr/bin/env python

class FStealerExploit:
	def initialize(self, target):
		raise Exception("Implement in inherited class.")

	def getFile(self, path):
		raise Exception("Implement in inherited class.")

class LocalStealerExploit(FStealerExploit):
	def initialize(self, target):
		pass

	def getFile(self, path):
		print "Getting file: %s"%path
		try:
			return open(path,"r").read()
		except IOError:
			return None



#!/usr/bin/env python

import re
from module import FStealerModule

class GenericModule(FStealerModule):
	def initialize(self):
		self.pathpattern = re.compile("((/[a-zA-Z0-9_\.]+)+)")

	def postGetFile(self, path, data):
		matches = map(lambda x: x[0], self.pathpattern.findall(data))
		for match in matches:
			self.addPath(match,1)
		self.tagPath(path, str(matches))

module = GenericModule



#!/usr/bin/env python

import os
import json

class ConfigurationException(Exception):
	pass

class StorageConfiguration(dict):
	MANDATORY_KEYS = [
						"WORKDIR",
						"BLACKLIST",
						"TAGFILE",
						"PATHFILE",
					]

	def __init__(self, filename = None, override = None):
		dict.__init__(self)

		if filename is not None:
			self._loadFrom(filename)

		if override is not None:
			self._fromHash(override)
		self._check()

	def _loadFrom(self, filename):
		f = open(filename, "r")
		h = json.load(f)
		self._fromHash(h)
		f.close()
	
	def _fromHash(self, override):
		for key,value in override.items():
			self[key] = value

	def _check(self):
		for key in self.MANDATORY_KEYS:
			if not self.has_key(key):
				raise ConfigurationException("Missing key: %s"%key)

class StorageDriver:
	def __init__(self, directory, blacklist):
		self.dir = os.path.abspath(directory)
		self.blacklist = blacklist
		self.fileIndex = {}
		self.buildIndex(self.dir, "")
		self.buildBlacklist("%s/%s"%(directory,"BLACKLIST"))
		self.tagFile = open("%s/%s"%(directory,"TAGFILE"), "a")

	def buildBlacklist(self, filename):
		try:
			f = open(filename, "r")
			for line in f:
				self.blacklist.add(line.strip())
			f.close()
		except IOError:
			pass
		self.blacklistFile = filename

	def buildIndex(self, directory, root):
		directory = os.path.realpath(directory)
		try:
			os.makedirs(directory)
		except:
			pass

		listing = os.listdir(directory)

		for item in listing:
			if os.path.isdir("%s/%s"%(directory,item)):
				self.buildIndex("%s/%s"%(directory,item),"%s/%s"%(root,item))
			else:
				self.fileIndex["%s/%s"%(root,item)] = "%s/%s"%(directory,item)
				self.blacklist.add("%s/%s"%(root,item))

	def __str__(self):
		s = ""
		for key,value in self.fileIndex.items():
			s+="%s:%s\n"%(key,value)
		return s

	def getRealPath(self, path):
		return "%s/%s"%(self.dir, path)

	def hasFile(self, path):
		if self.fileIndex.has_key(path):
			self.blacklist.add(path)
			return True
		return False

	def getFile(self, path):
		return open(self.fileIndex[path],"r").read()

	def putFile(self, path, data):
		realpath = self.getRealPath(path)
		try:
			os.makedirs(os.path.split(realpath)[0])
		except:
			pass
		self.fileIndex[path] = realpath

		f = open(realpath, "w")
		f.write(data)
		f.close()
		self.blacklist.add(path, noregex=True)

	def putTag(self, path, tag):
		self.tagFile.writelines("%s,%s\n"%(path,tag))

	def sync(self):
		self.tagFile.close()
		f = open(self.blacklistFile,"a")
		for path in self.blacklist:
			f.writelines("%s\n"%path.rawpattern)
		f.close()




#!/usr/bin/env python

import os
import exploit 
from store import StorageDriver
from blacklist import Blacklist
from pathlist import Pathlist
from mod import ModuleContainer,ModuleFactory

class ExploitFactory:
	def create(self, exploitclass, target):
		global exploit
		cls = getattr(exploit, exploitclass)
		exploit = cls()
		exploit.initialize(target)
		return exploit

class FileSystemStealer:
	def __init__(self, target, exploit, blacklist, odir, threadnum, initialpaths, moduledatapath):
		self.exploit = ExploitFactory().create(exploit, target)
		self.blacklist = Blacklist(blacklist)
		self.storageDriver = StorageDriver(odir, self.blacklist)
		self.pathlist = Pathlist(initialpaths, self.blacklist, odir)
		self.threadnum = threadnum
		self.modcontainer = ModuleContainer(ModuleFactory(self), moduledatapath)

	def run(self):
		try:
			while not self.pathlist.finished():
				path = self.pathlist.getNext()
				self.modcontainer.notify("preGetFile", path)
				data = self.exploit.getFile(path)
				if data is None:
					self.blacklistPath(path, noregex=True)
					continue
				self.storageDriver.putFile(path, data)
				self.modcontainer.notify("postGetFile", path, data)
		except KeyboardInterrupt:
			pass

		finally:
			self.sync()

	def sync(self):
		self.storageDriver.sync()
		self.pathlist.sync()
	
	### Module API implementation members following ###

	def addPath(self, path, priority):
		self.pathlist.addPath(path, priority)

	def delPath(self, path):
		self.pathlist.delPath(path)

	def blacklistPath(self, path, noregex=False):
		if noregex:
			self.blacklist.add("^%s$"%path)
		else:
			self.blacklist.add(path)

	def tagPath(self, path, tag):
		self.storageDriver.putTag(path, tag)



#!/usr/bin/env python

from libfstealer import *
import argparse
import sys

def parse_args():
	parser = argparse.ArgumentParser(description='fstealer - mirrors a filesystem via remote file disclosure')
	parser.add_argument('target', metavar='target', type=str, help='specify a target identifier to mirror from')
	parser.add_argument('--exploit', dest='exploit', default='LocalStealerExploit', help='specify the exploit to use, default is local reading')
	parser.add_argument('--blacklist', dest='blacklist', default='/etc/fstealer/blacklist', help='specify a file/path blacklist')
	parser.add_argument('--odir', dest='odir', default='/tmp/fstealer', help='specify the output directory')
	parser.add_argument('--threadnum', dest='threadnum', default='1', help='specify the number of threads to use')
	parser.add_argument('--pathlist', dest='pathlist', default='/etc/fstealer/pathlist', help='specify the initial list of paths')
	parser.add_argument('--module-storage', dest='moduledata', default='/etc/fstealer/moddata', help='specify the path to module specific file')
	return parser.parse_args()

if __name__ == "__main__":
	args = parse_args()
	fstealer = fsteal.FileSystemStealer(args.target, args.exploit, args.blacklist, args.odir, args.threadnum, args.pathlist, args.moduledata)
	fstealer.run()



#!/usr/bin/env python

import re
import os
from module import FStealerModule

class ApacheModule(FStealerModule):
	def initialize(self):
		self.apachepattern = re.compile("(.*/etc/apache2)")
		f = self.openModuleFile("signatures","r")
		self.apachesubpaths = []
		for line in f:
			self.apachesubpaths.append(line.strip())
		for subpath in self.apachesubpaths:
			self.addPath(os.path.join("/etc/apache2", subpath))

	def preGetFile(self, path):
		match = self.apachepattern.match(path)
		if match is not None:
			for subpath in self.apachesubpaths:
				self.addPath(os.path.join(match.group(0), subpath), priority=2)

module = ApacheModule



#!/usr/bin/env python

import re
import os
from module import FStealerModule

class ProcModule(FStealerModule):
	def initialize(self):
		f = self.openModuleFile("signatures","r")
		self.apachesubpaths = []
		for line in f:
			self.apachesubpaths.append(line.strip())
		for subpath in self.apachesubpaths:
			self.addPath(os.path.join("/proc", subpath))

module = ProcModule



#!/usr/bin/env python

import json

class Pathlist:
	DEFAULT_PRIORITY = 5
	persistent_pathlist = "PATHLIST"

	def __init__(self, pathfile, blacklist, workingdir, cont=True):
		self.paths = {}
		self.blacklist = blacklist
		self.workingdir = workingdir
		if not cont or not self.readStoredPaths():
			self.readInitialPaths(pathfile)

	def sync(self):
		self.writeStoredPaths()

	def readStoredPaths(self):
		try:
			f = open("%s/%s"%(self.workingdir, self.persistent_pathlist), "r")
			l = json.load(f)
			for path,priority in l:
				self.addPath(path,priority)

			return True
		except IOError:
			return False

	def writeStoredPaths(self):
		pl = []
		f = open("%s/%s"%(self.workingdir, self.persistent_pathlist), "w")
		for priority in self.paths:
			for path in self.paths[priority]:
				pl.append((path,priority))
		json.dump(pl, f)
		f.close()

	def readInitialPaths(self, pathfile):
		f = open(pathfile)
		for line in f:
			self.addPath(line.strip(), Pathlist.DEFAULT_PRIORITY)

	def getNext(self):
		return self.choosePath()

	def finished(self):
		for l in self.paths.values():
			if len(l) > 0:
				return False
		return True

	def addPath(self, path, priority):
		if self.blacklist.blacklisted(path):
			return

		if not self.paths.has_key(priority):
			self.paths[priority] = set()
		self.paths[priority].add(path)

	def delPath(self, path):
		for key in self.paths:
			self.paths[key].discard(path)

	def choosePath(self):
		keys = self.paths.keys()
		keys.sort()
		for key in keys:
			if len(self.paths[key]) > 0:
				return self.paths[key].pop()
		return None




import os
from modules import *

class ModuleFactory:
	def __init__(self, fstealer):
		self.fstealer = fstealer

	def create(self, cls, modulepath):
		return cls(self.fstealer, modulepath)

class ModuleContainer(list):
	def __init__(self, factory, moduledatapath):
		list.__init__(self)
		self.loadModules(factory, moduledatapath)

	def loadModules(self, factory, moduledatapath):
		lib = __import__("libfstealer")
		subdir = getattr(lib, "modules")
		modules = getattr(subdir, "__all__")
		for modulefile in modules:
			module = getattr(subdir, modulefile)
			module_class = getattr(module, "module")
			if module_class is not None:
				instance = factory.create(module_class, os.path.join(moduledatapath, module_class.__name__))
				self.append(instance)

	def notify(self, eventname, *args):
		for module in self:
			if hasattr(module, eventname):
				method = getattr(module, eventname)
				method(*args)
			



#!/usr/bin/env python

import re
from module import FStealerModule

class HomeModule(FStealerModule):
	suffixes = [
				".bash_history",
				".bash_logout",
				".bashrc",
				".xsession-errors",
				".viminfo",
				".vimrc",
				".Xmodmap",
				".profile",
				]

	def initialize(self):
		self.pathpattern = re.compile("^(/home/[^/]+)")

	def preGetFile(self, path):
		match = self.pathpattern.match(path)
		if match is not None:
			for suffix in self.suffixes:
				for p in match.groups():
					self.addPath("%s/%s"%(p,suffix), priority = 1)

module = HomeModule



#!/usr/bin/env python

import re

class BlacklistEntry:
	def __init__(self, pattern):
		self.rawpattern = pattern
		self.pattern = re.compile(pattern)

	def match(self, path):
		return self.pattern.match(path) is not None

	def __eq__(self, entry):
		return self.rawpattern == entry.rawpattern

	def __hash__(self):
		return hash(self.rawpattern)

class Blacklist(set):
	def __init__(self, blacklist):
		set.__init__(self)

		f = open(blacklist, "r")
		for line in f:
			self.add(line.strip())

	def blacklisted(self, path):
		for entry in self:
			if entry.match(path):
				return True
		return False

	def add(self, item, noregex = False):
		if noregex:
			set.add(self, BlacklistEntry("^%s$"%item))
		else:
			set.add(self, BlacklistEntry(item))





#!/usr/bin/env python

import os

class ModuleError(Exception):
	pass

class FStealerModule:
	def __init__(self, fstealer, modulepath):
		self.fstealer = fstealer
		self.modulepath = modulepath
		if hasattr(self, "initialize"):
			self.initialize()

	def addPath(self, path, priority=5):
		self.fstealer.addPath(path, priority)

	def delPath(self, path):
		self.fstealer.delPath(path)

	def blacklistPath(self, path):
		self.fstealer.blacklistPath(path)

	def tagPath(self, path, tag):
		self.fstealer.tagPath(path, tag)

	def openModuleFile(self, filename, mode):
		try:
			return open(os.path.join(self.modulepath, filename), mode)
		except IOError:
			raise ModuleError("Could not open module data: %s"%(filename))

module = None
